function [x,J] = adaptdesc(x,f,g,options,varargin)

% Steepest descent with adaptive stepsize
%
% f functional
% g gradient of f
% x variable
% in structure options:
%   maxiter = maximum number of iterations; default 500;
%   stepsize = initial stepsize of descent; default 1/norm(initial gradient)
%   stepmult = increase ratio of stepsize after each minimization success;
%     default 1.2
%   stepdiv = decrease ratio of stepsize after each minimization failure;
%     default 2
%   breakratio = ratio for termination criteria; default 1e-4
%   verbosemode if 1 then display evaluation of f at each step
%
% varargin additional arguments passed to f and g
%
% returns x variable at the end of minimization
% and J record of function evaluations
%
% author : Joan Glaunès - joan.glaunes@free.fr

maxiter = 500;
G = feval(g,x,varargin{:});
stepsize = 1/norm(G(:));
stepmult = 1.2;
stepdiv = 2;
breakratio = 1e-4;
verbosemode = 0;

loadstruct('options');

J(1) = feval(f,x,varargin{:});
for iter = 1:maxiter
    G = feval(g,x,varargin{:});
    stepsize = stepsize * stepdiv;
    minimtest = 0;
    while(~minimtest) 
        stepsize = stepsize / stepdiv;
        xnew = x - stepsize * G; 
        J(iter+1) = feval(f,xnew,varargin{:});
        minimtest = (J(iter+1) < J(iter));
    end
    x = xnew;
    stepsize = stepsize * stepmult;
    if verbosemode
        disp(['iteration ',num2str(iter),...
            '     functional = ',num2str(J(iter)),...
            '     stepsize = ', num2str(stepsize)]);
    end
    % termination criteria
    if (J(iter)-J(iter+1)) < breakratio*(J(1)-J(iter+1))
        return
    end
end

if verbosemode
    disp('maximum number of iterations exceeded');
end
